/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.oa.service;

import java.util.Date;
import java.util.List;

import com.jeeplus.modules.oa.entity.OaNotifyMaterial;
import com.jeeplus.modules.oa.mapper.OaNotifyMaterialMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.oa.entity.OaNotify;
import com.jeeplus.modules.oa.entity.OaNotifyRecord;
import com.jeeplus.modules.oa.mapper.OaNotifyMapper;
import com.jeeplus.modules.oa.mapper.OaNotifyRecordMapper;

/**
 * 通知通告Service
 * @author jeeplus
 * @version 2017-05-16
 */
@Service
@Transactional(readOnly = true)
public class OaNotifyService extends CrudService<OaNotifyMapper, OaNotify> {

	@Autowired
	private OaNotifyRecordMapper oaNotifyRecordMapper;

	@Autowired
	private OaNotifyMaterialMapper oaNotifyMaterialMapper;

	public OaNotify get(String id) {
		OaNotify entity = mapper.get(id);
		return entity;
	}
	
	/**
	 * 获取通知发送记录
	 * @param oaNotify
	 * @return
	 */
	public OaNotify getRecordList(OaNotify oaNotify) {
		oaNotify.setOaNotifyRecordList(oaNotifyRecordMapper.findList(new OaNotifyRecord(oaNotify)));
		return oaNotify;
	}
	
	public Page<OaNotify> find(Page<OaNotify> page, OaNotify oaNotify) {
		oaNotify.setPage(page);
		page.setList(mapper.findList(oaNotify));
		return page;
	}
	
	/**
	 * 获取通知数目
	 * @param oaNotify
	 * @return
	 */
	public Long findCount(OaNotify oaNotify) {
		return mapper.findCount(oaNotify);
	}
	
	@Transactional(readOnly = false)
	public void save(OaNotify oaNotify) {
		super.save(oaNotify);
		// 更新发送接受人记录
		oaNotifyRecordMapper.deleteByOaNotifyId(oaNotify.getId());
		if (oaNotify.getOaNotifyRecordList().size() > 0){
			oaNotifyRecordMapper.insertAll(oaNotify.getOaNotifyRecordList());
		}
	}

	@Transactional(readOnly = false)
	public void saveList(OaNotify oaNotify, List<OaNotifyMaterial> oaNotifyMaterials) {
		super.save(oaNotify);
		for (OaNotifyMaterial oaNotifyMaterial:oaNotifyMaterials){
			oaNotifyMaterial.preInsert();
			oaNotifyMaterial.setParentId(oaNotify.getId());
		}
		oaNotifyMaterialMapper.insertList(oaNotifyMaterials);
		// 更新发送接受人记录
		oaNotifyRecordMapper.deleteByOaNotifyId(oaNotify.getId());
		if (oaNotify.getOaNotifyRecordList().size() > 0){
			oaNotifyRecordMapper.insertAll(oaNotify.getOaNotifyRecordList());
		}
	}
	
	/**
	 * 更新阅读状态
	 */
	@Transactional(readOnly = false)
	public void updateReadFlag(OaNotify oaNotify) {
		OaNotifyRecord oaNotifyRecord = new OaNotifyRecord(oaNotify);
		oaNotifyRecord.setUser(oaNotifyRecord.getCurrentUser());
		oaNotifyRecord.setReadDate(new Date());
		oaNotifyRecord.setReadFlag("1");
		oaNotifyRecordMapper.update(oaNotifyRecord);
	}
}