/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.managementcenter.project.mapper;

import com.jeeplus.core.persistence.BaseMapper;
import com.jeeplus.core.persistence.annotation.MyBatisMapper;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.EngineeringAmount;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.OtherAmount;
import com.jeeplus.modules.sg.managementcenter.project.entity.PickingRequisition;
import com.jeeplus.modules.sg.managementcenter.project.entity.Project;
import com.jeeplus.modules.sg.managementcenter.project.entity.Project1;
import com.sun.tools.corba.se.idl.StringGen;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@MyBatisMapper
public interface ProjectMapper extends BaseMapper<Project> {

    /**
     * 修改项目前期基本信息
     */
    void insertBasic(Project project);

    /**
     * 导入批量项目前期基本信息
     */
    void insertBasicList(List<Project> project);

    /**
     * 导入批量项目前期基本信息
     */
    void insertAssList(List<Project> project);

    /**
     * 导入批量项目立项信息
     */
    void insertApprovalList(List<Project> project);

    /**
     * 批量修改导入的项目信息
     */
    void updateBasicList(List<Project> project);

    /**
     * 批量修改导入的施工任务分配信息
     */
    void updateAssList(List<Project> project);

    /**
     * 批量修改导入的项目立项信息
     */
    void updateApprovalList(List<Project> project);

    /**
     * 批量导入交底计划安排
     */
    void insertPlanList(List<Project> project);

    /**
     * 批量修改导入的交底计划安排
     */
    void updatePlanList(List<Project> project);

    /**
     * 项目现有项目所有的项目名称
     */
    List<String> allProjectName();

    /**
     * 项目现有项目所有的项目定义号
     */
    List<String> allProjectId();

    /**
     * 根据名称查询项目前期基本信息
     */
    Project findProByName(String name);

    /**
     * 根据名称查询施工任务分配信息
     */
    Project findAssByName(String name);


    /**
     * 根据名称查询交底计划安排信息
     */
    Project findPlanByName(String name);

    /**
     * 根据名称查询该计划的id，用于流程在表中添加procInsId
     * */
    String findPlanIdByName(String name);

    /**
     * 根据名称查询项目立项信息
     */
    Project findApprovalByName(String id);

    /**
     * 修改项目前期基本信息
     * */
    void updateBasic(Project project);

    /**
     * 施工任务分配信息数据
     */
    List<Project> findAssList(Project project);

    /**
     * 交底计划安排
     */
    List<Project> findPlanList(Project project);

    /**
     * 项目立项信息
     */
    List<Project> findApprovalList(Project project);

    /**
     * 项目名称查询项目信息
     * */
    Project findProcess(String name);

    /**
     * 根据年度物资上报id获取施工任务分配信息
     */
    Project getAss(Project project);

    /**
     * 根据年度物资上报id获取交底计划安排信息
     */
    Project getPlan(Project project);

    /**
     * 根据年度物资上报id获取项目立项信息信息
     */
    Project getApproval(Project project);



    /**
     * 查找系统中所有人员信息
     */
    List<String> findAllMember();

    /**
     * 获取需求单位
     */
    String findDemandUnit(String name);

    /**
     * 通过流程定义号获取需求单位
     */
    String demandByProcInsId(String procInsId);


    /**
     * 通过id获取项目暂定名称
     */
    String reserveProjectNameById(String id);


    Project findByProcInsId(String procInsId);

    /**
     * 获取项目基础信息根据id
     */
    Project findMaterialProject(String id);
    /*#################################################################################################*/



    /**
     * 根据改项目施工交底状态0未发起施工交底，1施工交底过程中，2完成施工交底
     * */
    void updateStatusBy(String acquisitionStatus,String projectId);

    void  updateStatusByClose(@Param("close") String close,@Param("procInsId") String procInsid);

    void updateAcquisitionStatus(@Param("acquisitionStatus")String acquisitionStatus,@Param("id") String id);

    /**
     * 根据id，查询所有项目信息
     * */
    List<Project> findListById(String[] projectIds);

    int insertList(@Param("list") List<Project1> list);

    List<Project> findListProject(List<Project1> projects);

    /**
     * 通过流程定实例id查询项目流转人员信息-施工交底流程
     * */
    Project findUser(String procInsId);

    Project findWZUser(String procInsId);

    Project findJSUser(String procInsId);

    Project sgFindUser(String procInsId);

    Project findUserById(String projectId);

    Integer updatePreConstruction(Project1 project);

    //查找项目信息
    List<Project> findListProjectRequisitions(@Param("list") List<PickingRequisition> list);
    //查找所有用户
    List<String> findAllUser();

    String findDefId(String key);

    Project1 findIdConProc(String procInsId);

    Project findProjectList(Project project);
}