/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.managementcenter.project.web;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.google.common.collect.Maps;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.act.service.ActProcessService;
import com.jeeplus.modules.act.service.ActTaskService;
import com.jeeplus.modules.oa.service.OaNotifyService;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.Construction;
import com.jeeplus.modules.sg.managementcenter.activiti.service.ConstructionService;
import com.jeeplus.modules.sg.managementcenter.materialInventory.service.MaterialInventoryService;
import com.jeeplus.modules.sg.managementcenter.project.entity.BasicInformation;
import com.jeeplus.modules.sg.managementcenter.project.entity.Project1;
import com.jeeplus.modules.sg.managementcenter.project.entity.Project;
import com.jeeplus.modules.sg.managementcenter.project.service.PickingRequisitionService;
import com.jeeplus.modules.sg.managementcenter.project.service.ProjectService;
import com.jeeplus.modules.sg.managementcenter.project.util.ProjectImportUtil;
import com.jeeplus.modules.sys.service.SystemService;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.activiti.engine.repository.ProcessDefinition;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;

/**
 * 项目基础信息controller
 *
 */
@Controller
@RequestMapping(value = "${adminPath}/sg/managementcenter/project")
public class ProjectController extends BaseController {

    @Autowired
    private ProjectService projectService;

    @Autowired
    private PickingRequisitionService pickingRequisitionService;

    @Autowired
    private MaterialInventoryService materialInventoryService;
    @Autowired
    private OaNotifyService oaNotifyService;//通知服务类

    @Autowired
    private SystemService systemService;

    @Autowired
    private ActProcessService actProcessService;

    @Autowired
    private ActTaskService actTaskService;

    @Autowired
    private ConstructionService constructionService;

    @ModelAttribute
    public Project get(@RequestParam(required = false) String id) {
        Project entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = projectService.get(id);
        }
        if (entity == null) {
            entity = new Project();
        }
        return entity;
    }

    /**
     * 工程管理表单列表页面
     */
    @RequiresPermissions("sg:managementcenter:project:basicInformation")
    @RequestMapping(value = "basicInformation")
    public String basicInformation(Project project, Model model) {
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/projectBasicInformation";
    }

    /**
     * 施工任务分配信息列表页面
     */
    @RequiresPermissions("sg:managementcenter:project:assignment")
    @RequestMapping(value = "assignment")
    public String assignment(Project project, Model model) {
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/projectAssignment";
    }

    /**
     * 交底计划安排
     */
    @RequiresPermissions("sg:managementcenter:project:disclosurePlan")
    @RequestMapping(value = "disclosurePlan")
    public String disclosurePlan(Project project, Model model) {
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/projectDisclosurePlan";
    }

    /**
     * 项目立项信息
     */
    @RequiresPermissions("sg:managementcenter:project:approval")
    @RequestMapping(value = "approval")
    public String approval(Project project, Model model) {
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/projectApproval";
    }

    /**
     * 项目前期基本信息 数据列表
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "basicData")
    public Map<String, Object> basicData(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        String reserveProjectName = project.getReserveProjectName();
        List<String> reserveList = new ArrayList<>();
        if (!"".equals(reserveProjectName)) {
            String[] split = reserveProjectName.split("\\s+");
            String reserveName = "";
            for (int i = 0; i < split.length; i++) {
                reserveName = split[i];
                reserveList.add(reserveName);
            }
            project.setReserveList(reserveList);
        }
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 项目前期基本信息 数据列表
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "assData")
    public Map<String, Object> assData(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        String reserveProjectName = project.getReserveProjectName();
        List<String> reserveList = new ArrayList<>();
        if (!"".equals(reserveProjectName)) {
            String[] split = reserveProjectName.split("\\s+");
            String reserveName = "";
            for (int i = 0; i < split.length; i++) {
                reserveName = split[i];
                reserveList.add(reserveName);
            }
            project.setReserveList(reserveList);
        }
        Page<Project> page = projectService.findAssPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 项目前期基本信息 数据列表
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "disclosureData")
    public Map<String, Object> disclosureData(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        String reserveProjectName = project.getReserveProjectName();
        List<String> reserveList = new ArrayList<>();
        if (!"".equals(reserveProjectName)) {
            String[] split = reserveProjectName.split("\\s+");
            String reserveName = "";
            for (int i = 0; i < split.length; i++) {
                reserveName = split[i];
                reserveList.add(reserveName);
            }
            project.setReserveList(reserveList);
        }
        Page<Project> page = projectService.findPlanPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 项目立项信息 数据列表
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "approvalData")
    public Map<String, Object> approvalData(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        String projectId = project.getProjectId();
        List<String> reserveList = new ArrayList<>();
        if (!"".equals(projectId)) {
            String[] split = projectId.split("\\s+");
            String reserveName = "";
            for (int i = 0; i < split.length; i++) {
                reserveName = split[i];
                reserveList.add(reserveName);
            }
            project.setReserveList(reserveList);
        }
        Page<Project> page = projectService.findApprovalPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 表单列表数据
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "data")
    public Map<String, Object> data(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 施工交底流程，项目选取框
     * */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:list")
    @RequestMapping(value = "dataSys")
    public Map<String, Object> dataSys(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
		project.setStatus("0");
        project.setWhetherCloseStatus("0");
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 领料模块项目选择框
     */
    @ResponseBody
    @RequestMapping(value = "dataPicking")
    public Map<String, Object> dataPicking(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
/*		project1.setStatus("2");
		project1.setWhetherCloseStatus("0");
		project1.setEarlyMaterial(1);
		project1.setConstructionUnitManage(UserUtils.getUser().getLoginName());*/
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 施工模块项目选择框
     */
    @ResponseBody
    @RequestMapping(value = "materialDataPicking")
    public Map<String, Object> materialDataPicking(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
/*		//运行提前施工
		project1.setPreConstruction(1);
		//未进行施工流程
		project1.setConstructionStatus("0");*/
        project.setStatus("2");
        project.setWhetherCloseStatus("0");
        project.setAdvanceStatus("0");
        project.setAdvanceConstruction("1");
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 提前施工项目选择
     */
    @ResponseBody
    @RequestMapping(value = "dataAdvance")
    public Map<String, Object> dataAdvance(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
/*		project1.setPreConstruction(0);
		project1.setConstructionUnitManage(UserUtils.getUser().getLoginName());//施工项目经理*/
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 领料申请项目选择框
     */
    @ResponseBody
    @RequestMapping(value = "pickingApplication")
    public Map<String, Object> pickingApplication(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 结算项目选择框
     */
    @ResponseBody
    @RequestMapping(value = "settData")
    public Map<String, Object> settData(Project project, HttpServletRequest request, HttpServletResponse response, Model model) {
        //project1.setSettStatus("0");
        //project1.setConstructionStatus("2");
        Page<Project> page = projectService.findPage(new Page<Project>(request, response), project);
        return getBootstrapData(page);
    }

    /**
     * 查看，增加，编辑表单表单页面
     */
    @RequiresPermissions(value = {"sg:managementcenter:project:view", "sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "basicInformationForm")
    public String basicInformationForm(Project project, Model model) {
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/basicInformationForm";
    }

    /**
     * 施工任务分配信息修改
     */
    @RequiresPermissions(value = {"sg:managementcenter:project:view", "sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "assignmentForm")
    public String assignmentForm(Project project, Model model) {
        project = projectService.getAss(project);
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/assignmentForm";
    }

    /**
     * 交底计划安排修改
     */
    @RequiresPermissions(value = {"sg:managementcenter:project:view", "sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "disclosurePlanForm")
    public String disclosurePlanForm(Project project, Model model) {
        project = projectService.getPlan(project);
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/disclosurePlanForm";
    }

    /**
     * 项目立项信息修改
     */
    @RequiresPermissions(value = {"sg:managementcenter:project:view", "sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "approvalForm")
    public String approvalForm(Project project, Model model) {
        project = projectService.getApproval(project);
        model.addAttribute("project", project);
        return "modules/sg/managementcenter/project/approvalForm";
    }


    /**
     * 保存表单
     */
    @ResponseBody
    @RequiresPermissions(value = {"sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "save")
    public AjaxJson save(Project project, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        Project projectStatus = projectService.get(project.getId());
        if ("0".equals(projectStatus.getStatus())) {
            BasicInformation basicInformation = project.getBasicInformation();
            String projectNature = basicInformation.getProjectNature();
            String demandUnit = basicInformation.getDemandUnit();
            String operationUnit = basicInformation.getOperationUnit();
            String powerSupplyArea = basicInformation.getPowerSupplyArea();
            String projectType = basicInformation.getProjectType();
            String whetherSummer = basicInformation.getWhetherSummer();
            if (!"资本".equals(projectNature) && !"成本".equals(projectNature)) {
                j.setSuccess(false);
                j.setMsg("项目性质填写有误");
                return j;
            }
            if (!"经研所".equals(demandUnit) && !"新区".equals(demandUnit) && !"丹徒".equals(demandUnit) && !"城区".equals(demandUnit)) {
                j.setSuccess(false);
                j.setMsg("需求单位填写有误");
                return j;
            }
            if (!"新区".equals(operationUnit) && !"丹徒".equals(operationUnit) && !"城区".equals(operationUnit)) {
                j.setSuccess(false);
                j.setMsg("项运维单位填写有误");
                return j;
            }
            if (!"".equals(powerSupplyArea)) {
                if (!"A".equals(powerSupplyArea) && !"A+".equals(powerSupplyArea) && !"B".equals(powerSupplyArea) && !"C".equals(powerSupplyArea) && !"无".equals(powerSupplyArea)) {
                    j.setSuccess(false);
                    j.setMsg("供电区域填写有误");
                    return j;
                }
            }
            if (!"配变改造".equals(projectType)
                    && !"配变新增".equals(projectType)
                    && !"电缆项目".equals(projectType)
                    && !"杆线项目".equals(projectType)
                    && !"混合项目".equals(projectType)
                    && !"纯自动化".equals(projectType)
                    && !"纯土建".equals(projectType)
                    && !"井盖维修".equals(projectType)
                    && !"环网柜更换".equals(projectType)
                    && !"箱变更换".equals(projectType)
                    && !"其他特殊类".equals(projectType)
            ) {
                j.setSuccess(false);
                j.setMsg("项目类型填写有误");
                return j;
            }
            if (!"是".equals(whetherSummer) && !"否".equals(whetherSummer)) {
                j.setSuccess(false);
                j.setMsg("是否迎峰度夏项目填写有误");
                return j;
            }
            Project proByName = projectService.findProByName(project.getReserveProjectName());
            //判断是不是空，是空insert新数据，不是空update数据
            if (proByName != null) {
                projectService.updateBasic(project);
            } else {
                project.getBasicInformation().setReserveProjectName(project.getReserveProjectName());
                projectService.insertBasic(project);
            }
            j.setSuccess(true);
            j.setMsg("保存成功");
        } else {
            j.setSuccess(false);
            j.setMsg("项目流程运行中不可修改信息");
        }
        return j;
    }

    /**
     * 保存表单
     */
    @ResponseBody
    @RequiresPermissions(value = {"sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "saveAss")
    public AjaxJson saveAss(Project project, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        Boolean flag = false;
        String str = "";
        /**
         * 后台hibernate-validation插件校验
         */
        Boolean aBoolean = ProjectImportUtil.checkProcess(project.getReserveProjectName());
        if (aBoolean) {
            String disclosureType = project.getAssignment().getDisclosureType();
            String assignedUnit = project.getAssignment().getAssignedUnit();
            String constructionUnit = project.getAssignment().getConstructionUnit();
            flag = ProjectImportUtil.checkFlag(disclosureType, ProjectImportUtil.DIS_TYPE);
            if (flag) {
                str += "交底类型填写有误\t";
            }
            flag = ProjectImportUtil.checkFlag(assignedUnit, ProjectImportUtil.ASS_UNIT);
            if (flag) {
                str += "合同单位填写有误\t";
            }
            flag = ProjectImportUtil.checkFlag(constructionUnit, ProjectImportUtil.CON_UNIT);
            if (flag) {
                str += "施工单位填写有误\t";
            }
            String s = ProjectImportUtil.checkMember(project);
            str += s;
            if (!"".equals(str)) {
                j.setSuccess(false);
                j.setMsg(str);
                return j;
            }
            Project proByName = projectService.findAssByName(project.getReserveProjectName());
            List<Project> projectList = new ArrayList<>();
            //判断是不是空，是空insert新数据，不是空update数据
            if (proByName != null) {
                projectList.add(project);
                projectService.updateAssList(projectList);
            } else {
                project.getAssignment().setReserveProjectName(project.getReserveProjectName());
                String demandUnit = projectService.findDemandUnit(project.getReserveProjectName());
                if (!"经研所".equals(demandUnit)) {
                    project.getAssignment().setInstituteRole("");
                }
                projectList.add(project);
                projectService.insertAssList(projectList);
            }
            j.setSuccess(true);
            j.setMsg("保存成功");
        } else {
            j.setSuccess(false);
            j.setMsg("项目流程运行中不可修改信息");
        }
        return j;
    }

    /**
     * 保存表单
     */
    @ResponseBody
    @RequiresPermissions(value = {"sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "savePlan")
    public AjaxJson savePlan(Project project, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        Boolean flag = false;
        String str = "";
        /**
         * 后台hibernate-validation插件校验
         */
        Boolean aBoolean = ProjectImportUtil.checkProcess(project.getReserveProjectName());
        if (aBoolean) {
            String reserveProjectName = project.getReserveProjectName();
            //判断项目前期基本信息-施工任务分配信息 是否填写完成，未完成不允许导入交底计划安排
            Project assByName = projectService.findAssByName(reserveProjectName);
            Project basicByName = projectService.findProByName(reserveProjectName);
            if (assByName == null || basicByName == null) {
                j.setSuccess(false);
                j.setMsg("请填写对应的前期基本信息/施工任务分配信息");
                return j;
            } else {
                project.setBasicInformation(basicByName.getBasicInformation());
                project.setAssignment(assByName.getAssignment());
                Project planByName = projectService.findPlanByName(reserveProjectName);
                List<Project> projectList = new ArrayList<>();
                //判断是不是空，是空insert新数据，不是空update数据
                projectList.add(project);
                if (planByName != null) {
//                    projectService.updatePlanList(projectList);
                    j.setSuccess(true);
                    j.setMsg("保存成功");
                } else {
                    projectService.insertPlanList(projectList);
                    Construction construction = new Construction();
                    construction.setReserveProjectName(project.getReserveProjectName());
                    constructionService.newSave(construction);
                    constructionService.updateStatus(project.getReserveProjectName());
                    Map<String, Object> vars = Maps.newHashMap();
                    vars.put("design", UserUtils.getByUserName(project.getAssignment().getConstructionManage()).getLoginName());
                    //发送流程，更改项目状态
                    String procDefId = projectService.findDefId("sgjd");
                    ProcessDefinition p = actProcessService.getProcessDefinition(procDefId);
                    String title = project.getCurrentUser().getName() + "在" + DateUtils.getDateTime() + "发起" + p.getName();
                    String ByName = projectService.findPlanIdByName(construction.getReserveProjectName());
                    actTaskService.startProcess(p.getKey(), "xm_construction_clarificaiton", ByName, title, vars);
                    j.setSuccess(true);
                    j.setMsg("流程发送成功");
                }
            }
        } else {
            j.setSuccess(false);
            j.setMsg("项目流程运行中不可修改信息");
        }
        return j;
    }

    /**
     * 保存表单
     */
    @ResponseBody
    @RequiresPermissions(value = {"sg:managementcenter:project:edit"}, logical = Logical.OR)
    @RequestMapping(value = "saveApproval")
    public AjaxJson saveApproval(Project project, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        Boolean flag = false;
        String str = "";
        /**
         * 后台hibernate-validation插件校验
         */
        String projectId = project.getProjectId();
        BigDecimal projectBudget = new BigDecimal(project.getApproval().getProjectBudget());
        String constructionCost = project.getApproval().getConstructionCost();
        String materialCost = project.getApproval().getMaterialCost();
        String designFee = project.getApproval().getDesignFee();
        String supervisionFee = project.getApproval().getSupervisionFee();
        String policyFee = project.getApproval().getPolicyFee();
        String otherExpenses = project.getApproval().getOtherExpenses();
        List<String> list = new ArrayList<>();
        if (!"".equals(constructionCost) && null != constructionCost) {
            list.add(constructionCost);
        }
        if (!"".equals(materialCost) && null != materialCost) {
            list.add(materialCost);
        }
        if (!"".equals(designFee) && null != designFee) {
            list.add(designFee);
        }

        if (!"".equals(supervisionFee) && null != supervisionFee) {
            list.add(supervisionFee);
        }
        if (!"".equals(policyFee) && null != policyFee) {
            list.add(policyFee);
        }
        if (!"".equals(otherExpenses) && null != otherExpenses) {
            list.add(otherExpenses);
        }
        BigDecimal totalAmount = new BigDecimal("0");
        for (String s : list) {
            totalAmount = totalAmount.add(new BigDecimal(s));
        }
        int i = projectBudget.compareTo(totalAmount);

        if (i != 0) {
            j.setSuccess(false);
            j.setMsg("合计金额不等于项目发文金额，请核实");
            return j;
        }

        Project planByName = projectService.findApprovalByName(projectId);
        List<Project> projectList = new ArrayList<>();
        //判断是不是空，是空insert新数据，不是空update数据
        projectList.add(project);
        if (planByName != null) {
            projectService.updateApprovalList(projectList);
        } else {
            projectService.insertApprovalList(projectList);
        }
        j.setSuccess(true);
        j.setMsg("保存成功");


        return j;
    }

    /**
     * 导出excel文件
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:export")
    @RequestMapping(value = "export")
    public AjaxJson exportFile(Project project, HttpServletRequest request, HttpServletResponse response) {
        AjaxJson j = new AjaxJson();

        try {
//			project1.setProjectManageRole(UserUtils.getUser().getLoginName());
            String fileName = "需求基本信息" + DateUtils.getDate("yyyyMMddHHmmss") + ".xlsx";

            HttpSession session = request.getSession();
//			Project projectEpo = (Project)session.getAttribute("projectEpo");

            Page<Project> page = projectService.findPage(new Page<Project>(request, response, -1), project);
            new ExportExcel("需求基本信息", Project.class).setDataList(page.getList()).write(response, fileName).dispose();
            j.setSuccess(true);
            j.setMsg("导出成功！");
            return j;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导出需求基本信息记录失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:import")
    @RequestMapping(value = "importBasic")
    public AjaxJson importBasic(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel importExcel = new ImportExcel(file, 1, "项目前期基本信息");
            int lastDataRowNum = importExcel.getLastDataRowNum();
            //将写入的文件先进行规则验证
            Map<String, Object> stringObjectMap = ProjectImportUtil.basicValidation(importExcel, lastDataRowNum);
            //获取错误信息
            String errorMsg = (String) stringObjectMap.get("errorMsg");
            //获取需要导入的list
            List<Project> afterValidation = (List<Project>) stringObjectMap.get("afterValidation");
            List<Project> noCompareList = (List<Project>) stringObjectMap.get("noCompareList");
            if (afterValidation.size() > 0) {
                projectService.insertBasicList(afterValidation);
            }
            if (noCompareList.size() > 0) {
                projectService.updateBasicList(noCompareList);
            }
            int i = afterValidation.size() + noCompareList.size();
            j.setMsg("已成功导入" + i + "条项目表单记录" + errorMsg);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入需求基本信息表单失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:import")
    @RequestMapping(value = "importAss")
    public AjaxJson importAss(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel importExcel = new ImportExcel(file, 1, "施工任务分配信息");
            int lastDataRowNum = importExcel.getLastDataRowNum();
            //将写入的文件先进行规则验证
            Map<String, Object> stringObjectMap = ProjectImportUtil.assValidation(importExcel, lastDataRowNum);
            //获取错误信息
            String errorMsg = (String) stringObjectMap.get("errorMsg");
            //获取需要导入的list
            List<Project> afterValidation = (List<Project>) stringObjectMap.get("afterValidation");
            List<Project> noCompareList = (List<Project>) stringObjectMap.get("noCompareList");
            if (afterValidation.size() > 0) {
                projectService.insertAssList(afterValidation);
            }
            if (noCompareList.size() > 0) {
                projectService.updateAssList(noCompareList);
            }
            int i = afterValidation.size() + noCompareList.size();
            j.setMsg("已成功导入" + i + "条记录\n" + errorMsg);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入需求基本信息表单失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:import")
    @RequestMapping(value = "importPlan")
    public AjaxJson importPlan(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel importExcel = new ImportExcel(file, 1, "交底计划安排");
            int lastDataRowNum = importExcel.getLastDataRowNum();
            //将写入的文件先进行规则验证
            Map<String, Object> stringObjectMap = ProjectImportUtil.planValidation(importExcel, lastDataRowNum);
            //获取错误信息
            String errorMsg = (String) stringObjectMap.get("errorMsg");
            //获取需要导入的list
            List<Project> afterValidation = (List<Project>) stringObjectMap.get("afterValidation");
            List<Project> noCompareList = (List<Project>) stringObjectMap.get("noCompareList");
            if (afterValidation.size() > 0) {
                projectService.insertPlanList(afterValidation);
            }
/*            if (noCompareList.size() > 0) {
                projectService.updatePlanList(noCompareList);
            }*/
            int i = afterValidation.size() + noCompareList.size();
            List<Project> processList = new ArrayList<>();
            if (afterValidation.size() > 0) {
                processList.addAll(afterValidation);
            }
            String procDefId = projectService.findDefId("sgjd");
            for (Project project : processList) {
                Construction construction = new Construction();
                construction.setReserveProjectName(project.getReserveProjectName());
                constructionService.newSave(construction);
                constructionService.updateStatus(project.getReserveProjectName());
                Map<String, Object> vars = Maps.newHashMap();
                vars.put("design", UserUtils.getByUserName(project.getAssignment().getConstructionManage()).getLoginName());
                //发送流程，更改项目状态
                ProcessDefinition p = actProcessService.getProcessDefinition(procDefId);
                String title = project.getCurrentUser().getName() + "在" + DateUtils.getDateTime() + "发起" + p.getName();
                String planByName = projectService.findPlanIdByName(construction.getReserveProjectName());
                actTaskService.startProcess(p.getKey(), "xm_construction_clarificaiton", planByName, title, vars);
            }
            //将不可修改的值重新获取
            j.setMsg("已成功导入" + i + "条记录\n" + errorMsg);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入需求基本信息表单失败！失败信息：" + e.getMessage());
        }
        return j;
    }

    /**
     * 导入Excel数据
     */
    @ResponseBody
    @RequiresPermissions("sg:managementcenter:project:import")
    @RequestMapping(value = "importApproval")
    public AjaxJson importApproval(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel importExcel = new ImportExcel(file, 1, "项目立项信息");
            int lastDataRowNum = importExcel.getLastDataRowNum();
            //将写入的文件先进行规则验证
            Map<String, Object> stringObjectMap = ProjectImportUtil.approvalValidation(importExcel, lastDataRowNum);
            //获取错误信息
            String errorMsg = (String) stringObjectMap.get("errorMsg");
            //获取需要导入的list
            List<Project> afterValidation = (List<Project>) stringObjectMap.get("afterValidation");
            List<Project> noCompareList = (List<Project>) stringObjectMap.get("noCompareList");
            if (afterValidation.size() > 0) {
                projectService.insertApprovalList(afterValidation);
            }
            if (noCompareList.size() > 0) {
                projectService.updateApprovalList(noCompareList);
            }
            int i = afterValidation.size() + noCompareList.size();
            List<Project> processList = new ArrayList<>();
            if (afterValidation.size() > 0) {
                processList.addAll(afterValidation);
            }
            if (noCompareList.size() > 0) {
                processList.addAll(noCompareList);
            }
            j.setMsg("已成功导入" + i + "条记录\n" + errorMsg);
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入需求基本信息表单失败！失败信息：" + e.getMessage());
        }
        return j;
    }
}