package com.jeeplus.modules.sg.materialmodule.activiti.web;

import com.google.common.collect.Maps;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.act.service.ActProcessService;
import com.jeeplus.modules.act.service.ActTaskService;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.Construction;
import com.jeeplus.modules.sg.managementcenter.activiti.service.ConstructionService;
import com.jeeplus.modules.sg.managementcenter.project.entity.Project;
import com.jeeplus.modules.sg.managementcenter.project.service.ProjectService;
import com.jeeplus.modules.sg.materialmodule.activiti.entity.MaterialModule;
import com.jeeplus.modules.sg.materialmodule.activiti.entity.NonPower;
import com.jeeplus.modules.sg.materialmodule.activiti.service.MaterialModuleService;
import com.jeeplus.modules.sg.materialmodule.activiti.service.NonPowerService;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.activiti.engine.repository.ProcessDefinition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;

@Controller
@RequestMapping(value = "${adminPath}/nonPower")
public class NonPowerController extends BaseController {
    @Autowired
    private NonPowerService nonPowerService;
    @Autowired
    private ActProcessService actProcessService;
    @Autowired
    private ActTaskService actTaskService;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private ConstructionService constructionService;

    @ModelAttribute
    public NonPower get(@RequestParam(required = false) String id) {
        NonPower entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = nonPowerService.get(id);
        }
        if (entity == null) {
            entity = new NonPower();
        }
        return entity;
    }


    /***************************************土建流程*************************************************/

    /**
     * 土建流程发起页面
     */
    @RequestMapping(value = "nonPowerForm")
    public String nonPowerForm(NonPower nonPower, Model model) {
        model.addAttribute("nonPower", nonPower);
        return "modules/sg/materialmodule/activiti/nonPowerForm";
    }

    /**
     * 土建开工流程
     */
    @RequestMapping(value = "nonPowerStart")
    public String nonPowerStart(NonPower nonPower, Model model) {
        model.addAttribute("nonPower", nonPower);
        return "modules/sg/materialmodule/activiti/nonPowerStart";
    }

    /**
     * 土建竣工
     */
    @RequestMapping(value = "nonPowerEnd")
    public String nonPowerEnd(NonPower nonPower, Model model) {
        model.addAttribute("nonPower", nonPower);
        return "modules/sg/materialmodule/activiti/nonPowerEnd";
    }

    /**
     * 在线验收
     */
    @RequestMapping(value = "acceptance")
    public String acceptance(NonPower nonPower, Model model) {
        model.addAttribute("nonPower", nonPower);
        return "modules/sg/materialmodule/activiti/acceptance";
    }

    /**
     * 审核土建竣工
     */
    @RequestMapping(value = "nonRePowerEnd")
    public String nonRePowerEnd(NonPower nonPower, Model model) {
        model.addAttribute("nonPower", nonPower);
        return "modules/sg/materialmodule/activiti/nonRePowerEnd";
    }

    /**
     * 发起土建流程
     */
    @ResponseBody
    @RequestMapping(value = "save")
    public AjaxJson save(NonPower nonPower, Model model) throws Exception {
        AjaxJson j = new AjaxJson();
        /**
         * 后台hibernate-validation插件校验
         */
        String errMsg = beanValidator(nonPower);
        if (StringUtils.isNotBlank(errMsg)) {
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        /**
         * 流程审批
         */
        if (StringUtils.isBlank(nonPower.getId())) {
            if ("".equals(nonPower.getProjectId())) {
                j.setSuccess(false);
                j.setMsg("请选择项目");
                return j;
            }
            Project project = new Project();
            project.setId(nonPower.getProjectId());
            //通过项目id获取项目定义号，项目名称及人员信息
            Project materialProject = projectService.getAss(project);
            //定义vars用于指派流程走向
            Map<String, Object> vars = Maps.newHashMap();
            //获取项目定义号
            String projectId = materialProject.getProjectId();
            vars.put("design", UserUtils.getByUserName(materialProject.getAssignment().getConstructionManage()).getLoginName());
            //保存基础信息到流程表中
            nonPower.setProjectId(projectId);
            nonPowerService.save(nonPower);
            //根据项目定义号修改项目的土建流程状态
            constructionService.updateAdvStatus(nonPower.getProjectId());
            // 启动流程
            ProcessDefinition p = actProcessService.getProcessDefinition(nonPower.getAct().getProcDefId());
            String title = nonPower.getCurrentUser().getName() + "在" + DateUtils.getDateTime() + "发起" + p.getName();
            actTaskService.startProcessNew(p.getKey(), "xm_nonpower_process", nonPower.getId(), title, vars);
            //新增或编辑表单保存
            j.setMsg("发起流程审批成功!");
            j.getBody().put("targetUrl", "/act/task/process/");
        }
        return j;
    }
    /***************************************土建流程*************************************************/
}


